
import java.util.Scanner;
/**
 * Created by leon on 2/9/18.
 * 
 * Expanded by Allison on 5/25/2018
 * 
 * The Console displays messages and fetches calculations for the user
 * while they are using the calculator. It uses the Calculator class to
 * make calculations, and the NumericInputParser to correctly read user
 * input and format calculations
 * 
 */
public class Console {

    private Calculator calc;
    private NumericInputParser parse;

    public Console(){
        calc = new Calculator();
        parse = new NumericInputParser();
    }

    public void runCommandLoop() 
    {
        Scanner input = new Scanner(System.in);
        println("Welcome to this calculator!");
        println("For a user guide, enter '?'");

        while (true) {
            print("Enter a command: ");
            String command = input.next();

            try {
                if (command.equalsIgnoreCase("exit"))  {
                    print("Thanks for calculating with us!");
                    break;
                }
                else if (command.equalsIgnoreCase("clear")){
                    calc.clear();
                    println(parse.getFormattedNumber(calc.getDisplayValue()));
                }
                else if (command.equalsIgnoreCase("val")){
                    double val = calc.getDisplayValue();
                    println("The current value is: %s", parse.getFormattedNumber(val));
                }
                else if (command.equalsIgnoreCase("?")){
                    printGuide();
                }

                //Core Features

                else if (command.equalsIgnoreCase("add"))  {
                    double d1 = getDoubleInput("First value: ");
                    double d2 = getDoubleInput("Second value: ");
                    double result = calc.add(d1, d2);
                    println("The sum is: %s", parse.getFormattedNumber(result));
                }
                else if (command.equalsIgnoreCase("sub")) {
                    double d1 = getDoubleInput("First value: ");
                    double d2 = getDoubleInput("Second value: ");
                    double result = calc.subtract(d1, d2);
                    println("The difference is: %s", parse.getFormattedNumber(result));

                }
                else if (command.equalsIgnoreCase("mult")) {
                    double d1 = getDoubleInput("Multiplicand: ");
                    double d2 = getDoubleInput("Multiplier: ");
                    double result = calc.multiply(d1, d2);
                    println("The product is: %s", parse.getFormattedNumber(result));
                }
                else if (command.equalsIgnoreCase("div")) {
                    double d1 = getDoubleInput("Dividend: ");
                    double d2 = getDoubleInput("Divisor: ");
                    if (d2 == 0){
                        errMessage();
                    }
                    else { 
                        double result = calc.divide(d1, d2);
                        println("The quotient is: %s", parse.getFormattedNumber(result));
                    }   
                }
                else if (command.equalsIgnoreCase("sq"))  {
                    double d = getDoubleInput("Value: ");
                    double result = calc.square(d);
                    println("The square is: %s", parse.getFormattedNumber(result));

                }
                else if (command.equalsIgnoreCase("pow")) {
                    double b = getDoubleInput("Base: ");
                    int e = getIntegerInput("Exponent: ");
                    double result = calc.pow(b, e);
                    println("The exponentiation is: %s", parse.getFormattedNumber(result));
                }
                else if (command.equalsIgnoreCase("sqrt")) {
                    double d = getDoubleInput("Value: ");
                    double result = calc.squareRoot(d);
                    println("The square root is: %s", parse.getFormattedNumber(result));

                }
                else if (command.equalsIgnoreCase("inv")) {
                    double d = getDoubleInput("Value: ");
                    if (d == 0){
                        errMessage();
                    }
                    else {
                        double result = calc.inverse(d);
                        println("The inverse is: %s", result);
                    }
                }

                //trig functions
                else if (command.equalsIgnoreCase("sin")) {
                    double d = getDoubleInput("Value: ");
                    //call the sin method on calc and display
                    double result = calc.sin(d);
                    println("The sin is: %s", parse.getFormattedAngle(result));

                }
                else if (command.equalsIgnoreCase("cos")) {
                    double d = getDoubleInput("Value: ");
                    //call the sin method on calc and display
                    double result = calc.cos(d);
                    println("The cos is: %s", parse.getFormattedAngle(result));

                }
                else if (command.equalsIgnoreCase("tan")) {
                    double d = getDoubleInput("Value: ");
                    //call the tan method on calc and display
                    double result = calc.tan(d);
                    println("The tan is: %s", parse.getFormattedAngle(result));

                }
                else if (command.equalsIgnoreCase("cot")) {
                    double d = getDoubleInput("Value: ");
                    double result = calc.inverseTan(d);
                    println("The cot is: %s", parse.getFormattedAngle(result));

                }
                else if (command.equalsIgnoreCase("csc")){
                    double d = getDoubleInput("Value: ");
                    double result = calc.inverseSin(d);
                    println("The csc is: %s", parse.getFormattedAngle(result));
                }
                else if (command.equalsIgnoreCase("sec")){
                    double d = getDoubleInput("Value: ");
                    double result = calc.inverseCos(d);
                    println("The sec is: %s", parse.getFormattedAngle(result));
                }

                //Bonus Functions
                else if (command.equalsIgnoreCase("log")){
                    double d = getDoubleInput("Value: ");
                    double result = calc.log(d);
                    println("The log is: %s", parse.getFormattedNumber(result));
                }
                else if (command.equalsIgnoreCase("antilog")){
                    double d = getDoubleInput("Value: ");
                    double result = calc.inverseLog(d);
                    println("The inverse log (base 10) is: %s", parse.getFormattedNumber(result));
                }
                else if (command.equalsIgnoreCase("antiln")){
                    double d = getDoubleInput("Value: ");
                    double result = calc.inverseLn(d);
                    println("The inverse natural log is: %s", parse.getFormattedNumber(result));
                }
                else if (command.equalsIgnoreCase("pi")){
                    println("The current value is now: %s", parse.getFormattedNumber(calc.displayPi()));
                }
                else if (command.equalsIgnoreCase("e")){
                    println("The current value is now: %s", parse.getFormattedNumber(calc.displayE()));
                }
                //display mode changes
                else if (command.equalsIgnoreCase("hex") ||
                command.equalsIgnoreCase("oct") ||
                command.equalsIgnoreCase("dec") ||
                command.equalsIgnoreCase("bin") ){
                    String displayMode = parse.switchDisplayMode(command);
                    println("Display mode changed to %s", displayMode);
                }
                else if (command.equalsIgnoreCase("display")) {
                    String displayMode = parse.switchDisplayMode();
                    println("Display mode changed to %s", displayMode);
                }

                //trig units mode changes
                else if (command.equalsIgnoreCase("rad") ||
                command.equalsIgnoreCase("deg")) {
                    String trigUnitsMode = parse.switchUnitsMode(command);
                    println("Trig units changed to %s", trigUnitsMode);
                }
                else if (command.equalsIgnoreCase("trig")) {
                    String trigUnitsMode = parse.switchUnitsMode();
                    println("Trig units changed to %s", trigUnitsMode);
                }

                //memory features
                else if (command.equalsIgnoreCase("M+")) {
                    //add the display value to memory
                    double mem = calc.memSet();
                    println("Memory value set to: %s", parse.getFormattedNumber(mem));
                }
                else if (command.equalsIgnoreCase("MC")) {
                    //clear the memory
                    double mem = calc.memClear();
                    println("Memory cleared.\nMemory value set to: %s", parse.getFormattedNumber(mem));
                }
                else if (command.equalsIgnoreCase("MRC")) {
                    double mem = calc.mrc();

                }

                else {
                    System.out.println("Invalid input. Enter '?' for a user guide");
                    System.out.println("Enter 'exit' to quit");
                }
            }
            catch (ArithmeticException e) {
                errMessage();
            }
        }

    }

    public static void print(String output, Object... args) {
        System.out.printf(output, args);
    }

    public static void println(String output, Object... args) {
        print(output + "\n", args);
    }

    public String getStringInput(String prompt) {
        Scanner s = new Scanner(System.in);
        print(prompt);
        return s.nextLine();
    }

    public Integer getIntegerInput(String prompt) {
        Scanner s = new Scanner(System.in);
        print(prompt);
        Integer i = null;
        try {
            i = parse.getInt(s.nextLine());
        }
        catch (NumberFormatException e) {
            boolean validInputFound = false;
            println("ERROR. Invalid input.");
            print(prompt);
            while (!validInputFound) {
                print(prompt);
                try {
                    i = parse.getInt(s.nextLine());
                    validInputFound = true;
                }
                catch (NumberFormatException e2){
                    println("ERROR. Invalid input.");
                    println("Current display mod is: " + parse.getTrigUnitsMode());
                }
            }
        }
        return i;

    }

    public Double getDoubleInput(String prompt) {
        Scanner s = new Scanner(System.in);
        print(prompt);
        Double d = null;
        try {
            d = parse.getDouble(s.nextLine());
        }
        catch (NumberFormatException e) {
            boolean validInputFound = false;
            println("ERROR. Invalid input.");
            while (!validInputFound) {
                print(prompt);
                try {
                    d = parse.getDouble(s.nextLine());
                    validInputFound = true;
                }
                catch (NumberFormatException e2){
                    println("ERROR. Invalid input.");
                    println("Current display mode is: " + parse.getDisplayMode());
                }
            }

        }
        return d;
    }

    private void errMessage(){
        while(true){
            String clear = getStringInput("ERR\n");
            if (clear.equalsIgnoreCase("clear")){
                break;
            }   
        }
    }

    public double getAngleInput(String prompt) {
        Scanner s = new Scanner(System.in);
        print(prompt);
        return parse.getAngle(s.nextLine());
    }

    public void printGuide(){
        println("Guide Under Construction");
    }
}
